nextflow.enable.dsl = 2

params.ped_file = null
params.sample_sheet = null

workflow read_inputs {
    /*
    Read the input files into channels. This has to be a workflow so the channels
    can be included in other scripts
    */

    main:
        /*
        ch_family_info - tuple channel of family members:
        [
            [
                family: family1,
                indv: indv1,
                father: indv2,
                mother: indv3,
                sex: 2,
                affected: 2
            ],
            ...
        ]
        */
        ped_file = file(params.ped_file, checkIfExists: true)
        ch_ped_file_info = Channel.fromPath(ped_file)
            .splitCsv(sep: '\t')
            .map(
                { line ->
                    [
                        line[1],  // indv ID
                        line[0],  // family ID
                        line[2],  // father
                        line[3],  // mother
                        line[4],  // sex
                        line[5]   // affected
                    ]
                }
            )


        /*
        ch_samplesheet_info - Tuple channel of individual ID, r1 fastqs and r2 fastqs:
        [
            [
                indv1,
                [/abs/path/to/lane_1_r1.fastq.gz, /abs/path/to/lane_2_r1.fastq.gz],
                [/abs/path/to/lane_1_r2.fastq.gz, /abs/path/to/lane_2_r2.fastq.gz]
            ],
            [
                indv2,
                ...
            ]
        ]
        */
        samplesheet = file(params.sample_sheet, checkIfExists: true)
        ch_samplesheet_info = Channel.fromPath(samplesheet)
            .splitCsv(sep:'\t', header: true)
            .map(
                { line -> [line.individual_id, file(line.read_1), file(line.read_2)] }
            )
            .groupTuple()

        /*
        ch_individuals_by_family - merge of samplesheet and ped file information,
        grouped by family ID:
        [
            [
                indv1,
                [
                    indv_id,
                    family_id,
                    father_id,
                    mother_id,
                    sex,
                    affected,
                    r1_fastqs,
                    r2_fastqs
                ]
            ],
            ...
        ]
        */
        ch_individuals_by_family = ch_ped_file_info
            .join(ch_samplesheet_info)
            .map({[it[1], it]})
    
    emit:
        ch_samplesheet_info
        ch_ped_file_info
        ch_individuals_by_family
}
